using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Reflection;

namespace MVRPlugin 
{
	/// <summary>
	/// Plugin Settings Script for Dark Theme
	/// </summary>
    public class SettingsDarkTheme : MVRScript
    {	
		private static MVRScript              script;	
		private static Color                  colorGray_17_Percent   = GrayShade(  45f );
        private static Color                  colorGray_26_Percent   = GrayShade(  68f );
        private static Color                  colorGray_31_Percent   = GrayShade(  80f );
        private static Color                  colorGray_36_Percent   = GrayShade(  92f );
        private static Color                  colorGray_59_Percent   = GrayShade( 152f );
        private static Color                  colorGray_66_Percent   = GrayShade( 170f );
        private static Color                  colorGray_80_Percent   = GrayShade( 205f );
        private static Color                  colorGray_92_Percent   = GrayShade( 235f );							   
        private static Color                  darkRed                = GetColor(  119f, 95f,  95f);
        private static Color                  darkBlue               = GetColor(  95f,  95f, 119f);
		public  static Color                  colorTextNormal        = colorGray_59_Percent;
        public  static Color                  colorTextLighter       = colorGray_80_Percent;		
		private static Color                  colorSettingNormal     = Color.white;
		private static Color                  colorSettingActive     = colorGray_59_Percent;

		private static readonly int           colorSettings          = 53;
		private static string[]               colorLabels            = new string[colorSettings];
		private static JSONStorableColor[]    colorStorables         = new JSONStorableColor[colorSettings];
		private static UIDynamicButton[]      colorDescriptions      = new UIDynamicButton[colorSettings];
		private static UIDynamicButton[]      colorButtons           = new UIDynamicButton[colorSettings];
		private static UIDynamicColorPicker   ColorPicker            = null;		
		private static int                    currentIndex           = -1;
																     
		public  static Color[]                Colors                 = new Color[colorSettings];
		
		
		/// <summary>
		/// Update color from external Plugin
		/// </summary>
		public static void UpdateColors(Color[] ColorsToSet)
		{
			// Set default colors
			Colors = ColorsToSet;  
			
			// If colors are not setup correctly, use default colors
			if (Colors.Length != 53)
			{ 
				ResetAllColors(); 
				return;
			}
			
			// Set colors from external Plugin
			for (int index = 0; index < Colors.Length; index++)
			{ 
				if (colorButtons[index] != null)
				{ colorButtons[index].buttonColor = Colors[index]; }
			}
		}
		
		/// <summary>
		/// Initialize MVRScript and UI
		/// </summary>
        public override void Init()
        {
			try
            {
				// Make sure, that Plugin was called correctly
				if (containingAtom.type != "CoreControl" && containingAtom.type != "SessionPluginManager")
                {
                    SuperController.LogError($"Please load DarkTheme as a Session or Scene Plugin, not with a '{containingAtom.type}' atom.");
                    DestroyImmediate(this);
                    return;
                }
				
				// Add Caption
				SetupInfoText(this, "<color=#606060><size=40><b>Dark Theme</b></size></color>", 20.0f, false);
				
				// Add informational Notification
				SetupInfoText(this,
                    "<color=#606060><size=40><b>Saving Settings</b></size>\n" +
                    "To permanently save the settings below when restarting Virt-a-Mate, " +
                    "make sure to choose 'Session Plugin Presets -> Change User Defaults -> Set Current As User Defaults'.</color>\n\n",
                    270.0f, false
                );	
				
				// Store Script reference
				script = this;

				// Initialization: Set labels
				SetLabels();

				// Create Color Picker
				JSONStorableColor storable = new JSONStorableColor("Pick color to change", GetHSVFromColor(Color.white));
				ColorPicker                = CreateColorPicker(storable, true);								
				ColorPicker.colorPicker.onColorChangedHandlers = new HSVColorPicker.OnColorChanged(OnColorChanged);
				
				// Create Spacer: Left
				var spacerLeft    = CreateSpacer(false);
				spacerLeft.height = 1f;
				
				// Create Spacer: Right
				var spacerRight    = CreateSpacer(true);
				spacerRight.height = 5f;
				
				// Create button to reset all colors
				UIDynamicButton buttonResetColors   = this.CreateButton("Reset all Colors", false);
				buttonResetColors.height            = 40f;
				buttonResetColors.button.onClick.AddListener(() => ResetAllColors());
				
				// Create Spacer: Right
				var spacerResetAllColors    = CreateSpacer(true);
				spacerResetAllColors.height = 51f;
				
				// Create Color Settings
				for (int index = 0; index < colorSettings; index++)
				{ CreateColorRow(this, index, colorLabels[index], Colors[index]); }
            }
            catch (Exception e)
            { SuperController.LogError("Failed to initialize plugin: " + e); }
        }
		
		/// <summary>
		/// Create Color Setting
		/// </summary>
		private static void CreateColorRow(MVRScript script, int Index, string SettingDescription, Color DefaultColor)
        {
			// Create button on the left with the name of the color setting
			UIDynamicButton buttonDescriptions   = script.CreateButton(SettingDescription, false);
			buttonDescriptions.height            = 40f;
			colorDescriptions[Index]             = buttonDescriptions.GetComponent<UIDynamicButton>();
			colorDescriptions[Index].buttonColor = colorSettingNormal;			
			colorDescriptions[Index].button.onClick.AddListener(() => ChangeColor(Index));
			
			// Align Color Description text on the left
			buttonDescriptions.buttonText.alignment = TextAnchor.MiddleLeft;
			
			// Create button on the right with the shown color of the color setting			
			UIDynamicButton buttonColor          = script.CreateButton(string.Empty, true);
			buttonColor.height                   = 40f;
			buttonColor.buttonColor              = DefaultColor;
			buttonColor.button.onClick.AddListener(() => ChangeColor(Index));
			colorButtons[Index]                  = buttonColor;
		}
		
		/// <summary>
		/// Change Color 
		/// </summary>
		private static void ChangeColor(int Index)
        {
			// Change background color of previous Color back to normal
			if (currentIndex != -1)
			{ colorDescriptions[currentIndex].buttonColor = colorSettingNormal; }
			
			// Store color index that is currently processed
			currentIndex = Index;
			
			// Change background color of color that is being changed
			colorDescriptions[Index].buttonColor = colorSettingActive;
			
			// Change label of Color Picker to color that is currently changed
			ColorPicker.label = colorDescriptions[Index].label;
			
			// Set color of Color Picker to color that was picked
			ColorPicker.colorPicker.SetHSV(GetHSVFromColor(colorButtons[Index].buttonColor));
		}
		
		/// <summary>
		/// When the user changes the color in the color picker, update the color in the settings row
		/// </summary>
		private static void OnColorChanged(Color ChangedColor)
        {
			// Make sure, a color was selected
			if (currentIndex == -1)
			{ return; }
			
			// Update the color in the settings row
			colorButtons[currentIndex].buttonColor = ChangedColor;
			
			// Change color
			Colors[currentIndex] = ChangedColor; 
			
			// Change color in Storable
			colorStorables[currentIndex].SetColor(ChangedColor); 
		}	
		
		/// <summary>
		/// Color -> HSVColor
		/// </summary>
		private static HSVColor GetHSVFromColor(Color Color)
		{ return HSVColorPicker.RGBToHSV(Color.r, Color.g, Color.b); }
		
		/// <summary>
		/// Color -> HSVColor
		/// </summary>
		private static Color GetColorFromHSV(HSVColor Color)
		{ return HSVColorPicker.HSVToRGB(Color.H, Color.S, Color.V); }
		
		/// <summary>
		/// HSVColor -> Color
		/// </summary>
		private static Color GrayShade(float ColorValue)
        { return new Color(ColorValue / 256f, ColorValue / 256f, ColorValue / 256f); }
		
		/// <summary>
        /// Get Color in range: [0..255] 
        /// </summary>
        private static Color GetColor(float Red, float Green, float Blue)
        { return new Color(Red / 256f, Green / 256f, Blue / 256f); }
		
		/// <summary>
		/// Show informational text
		/// </summary>
		private static JSONStorableString SetupInfoText(MVRScript script, string text, float height, bool rightSide)
        {
            JSONStorableString storable = new JSONStorableString("Info", text);
            UIDynamic textfield = script.CreateTextField(storable, rightSide);
            textfield.height = height;
            return storable;
        }
		
		/// <summary>
		/// Reset all colors
		/// </summary>
		private static void SetLabels()
		{
			colorLabels[0]  = "Text: Normal";
			colorLabels[1]  = "Text: Lighter (for darker backgrounds)";
			colorLabels[2]  = "Background";
			colorLabels[3]  = "Tab Backgrounds";
			colorLabels[4]  = "User Preferences: Tab Backgrounds";
			colorLabels[5]  = "User Preferences: VR 2 Tab panels";
			colorLabels[6]  = "Scene Animation Tab Background";
			colorLabels[7]  = "Version bar (Freeze Motion/Sound) bg.";
			colorLabels[8]  = "File Button: Background";
			colorLabels[9]  = "File Button: Image loading";
			colorLabels[10] = "File Button: Title Text";
			colorLabels[11] = "Hub Browser: Thumbnail loading";			
			colorLabels[12] = "Atom UI: Tab Group 1";
			colorLabels[13] = "Atom UI: Tab Group 2";
			colorLabels[14] = "Atom UI: Tab Group 3";
			colorLabels[15] = "Atom UI: Tab Group 4";
			colorLabels[16] = "Atom UI: Tab Group 5";
			colorLabels[17] = "Atom UI: Tab Group 6";
			colorLabels[18] = "Atom UI: Auto Behavior/Animation Tabs";
			colorLabels[19] = "Atom UI: Auto Beh./Anim.: Active Tab";
			colorLabels[20] = "Settings: Tab Group 1";
			colorLabels[21] = "Settings: Tab Group 2";
			colorLabels[22] = "Settings: Tab Group 3";
			colorLabels[23] = "Settings: Tab Group 4";
			colorLabels[24] = "User Preferences: Performance Tabs";
			colorLabels[25] = "User Preferences: VR Tabs";
			colorLabels[26] = "User Preferences: Misc / Security Tabs";
			colorLabels[27] = "Tab Group Texts";
			colorLabels[28] = "Clothing Item background";
			colorLabels[29] = "Hair Item background";
			colorLabels[30] = "Morph (in package)";
			colorLabels[31] = "Morph (not in package)";
			colorLabels[32] = "Morph Name Text";
			colorLabels[33] = "Button: Normal";
			colorLabels[34] = "Button: Highlighted";
			colorLabels[35] = "Button: Pressed";
			colorLabels[36] = "Button: Disabled";
			colorLabels[37] = "Toggle: Background";
			colorLabels[38] = "Toggle: Background Checkmark";
			colorLabels[39] = "Toggle: Text";
			colorLabels[40] = "Input Field: Background";			
			colorLabels[41] = "Input Field: Entered Text";
			colorLabels[42] = "Input Field: Placeholder Text";
			colorLabels[43] = "Input Field: X-Close Button";
			colorLabels[44] = "Input Field: X-Button Background";
			colorLabels[45] = "Slider: Background";
			colorLabels[46] = "Slider: Track";
			colorLabels[47] = "Slider: Bar";
			colorLabels[48] = "Slider: Handle";
			colorLabels[49] = "Slider: Text Name";
			colorLabels[50] = "Slider: Text Value";
			colorLabels[51] = "Scrollbars: Background";
			colorLabels[52] = "Scrollbars: Handle";
		}
		
		/// <summary>
		/// Reset all colors
		/// </summary>
		private static void ResetAllColors()
		{
			// Set default colors
			Colors[0]  = colorGray_59_Percent;   
			Colors[1]  = colorGray_80_Percent;   
			Colors[2]  = colorGray_36_Percent;   
			Colors[3]  = colorGray_36_Percent;   
			Colors[4]  = colorGray_31_Percent;   
			Colors[5]  = darkBlue;               
			Colors[6]  = colorGray_31_Percent;   
			Colors[7]  = colorGray_17_Percent;   
			Colors[8]  = colorGray_17_Percent;   
			Colors[9]  = colorGray_26_Percent;   
			Colors[10] = colorTextNormal;        
			Colors[11] = colorGray_36_Percent;   
			Colors[12] = colorGray_26_Percent;   
			Colors[13] = colorGray_36_Percent;   
			Colors[14] = colorGray_26_Percent;   
			Colors[15] = colorGray_36_Percent;   
			Colors[16] = colorGray_26_Percent;   
			Colors[17] = colorGray_36_Percent;   
			Colors[18] = colorGray_26_Percent;   
			Colors[19] = darkRed;                
			Colors[20] = colorGray_26_Percent;   
			Colors[21] = colorGray_36_Percent;   
			Colors[22] = colorGray_26_Percent;   
			Colors[23] = colorGray_36_Percent;   
			Colors[24] = colorGray_26_Percent;   
			Colors[25] = colorGray_26_Percent;   
			Colors[26] = colorGray_26_Percent;   
			Colors[27] = colorTextLighter;       
			Colors[28] = colorGray_80_Percent;   
			Colors[29] = colorGray_80_Percent;   
			Colors[30] = darkRed;                
			Colors[31] = darkBlue;               
			Colors[32] = colorTextLighter;       
			Colors[33] = colorGray_80_Percent;   
			Colors[34] = colorGray_92_Percent;   
			Colors[35] = colorGray_66_Percent;   
			Colors[36] = colorGray_66_Percent;   
			Colors[37] = colorGray_36_Percent;   
			Colors[38] = colorGray_80_Percent;   
			Colors[39] = colorTextLighter;       
			Colors[40] = colorGray_36_Percent;   
			Colors[41] = colorTextLighter;       
			Colors[42] = colorTextLighter;       
			Colors[43] = colorGray_59_Percent;   
			Colors[44] = colorGray_59_Percent;   
			Colors[45] = colorGray_26_Percent;   
			Colors[46] = colorGray_36_Percent;   
			Colors[47] = colorGray_66_Percent;   
			Colors[48] = colorGray_80_Percent;   
			Colors[49] = colorTextNormal;        
			Colors[50] = colorTextNormal;        
			Colors[51] = colorGray_36_Percent;   
			Colors[52] = colorTextNormal; 
			
			// Set colors from external Plugin
			for (int index = 0; index < Colors.Length; index++)
			{ 
				if (colorButtons[index] != null)
				{ colorButtons[index].buttonColor = Colors[index]; }
			}
		}	
    }
}
